/*
 * This file is part of VLE, a framework for multi-modeling, simulation and
 * analysis of complex dynamical systems
 * http://www.vle-project.org
 *
 * Copyright (c) 2003-2007 Gauthier Quesnel <quesnel@users.sourceforge.net>
 * Copyright (c) 2003-2010 ULCO http://www.univ-littoral.fr
 * Copyright (c) 2007-2010 INRA http://www.inra.fr
 *
 * See the AUTHORS or Authors.txt file for copyright owners and contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <vle/extension/DifferentialEquation.hpp>
#include <iostream>

namespace vd = vle::devs;
namespace vv = vle::value;
namespace ve = vle::extension;
namespace ved = vle::extension::differential_equation;
namespace vu = vle::utils;

namespace Multiformalism {

class LotkaVolterraQSSmy : public ved::DifferentialEquation
{
public:
    LotkaVolterraQSSmy(const vd::DynamicsInit& model, const vd::InitEventList& events) : 
        ved::DifferentialEquation(model,events)
    {
        // Lecture des valeurs de parametres dans les conditions du vpz
        // Ces parametres ont une valeur par defaut utilise si la condition n'est pas definie
        delta = (events.exist("delta")) ? vv::toDouble(events.get("delta")) : 0.5;
        gamma = (events.exist("gamma")) ? vv::toDouble(events.get("gamma")) : 0.5;

        log = (events.exist("log")) ? vv::toBoolean(events.get("log")) : false;

        // Variables d'etat gerees par ce composant
        y = createVar("y");

        x = createExt("x");
    }

    virtual ~LotkaVolterraQSSmy() {}

    void compute(const vle::devs::Time& time)
    {
        if (log)
            std::cout << time << " QSSMy x() :" <<x() << " QSSMy y() :" << y() << std::endl;
        grad(y) = -y() * (delta - gamma * x());
    }

private :
    //paramètres
    double delta; // taux de mortalité des prédateurs en l’absence de proies
    double gamma; // taux de reproduction des prédateurs en fonction des proies mangées

    bool log;

    //variables d'etat
    Var y; // effectif des prédateurs en fonction du temps

    Ext x;

};

} // namespace Multiformalism

DECLARE_DYNAMICS(Multiformalism::LotkaVolterraQSSmy)
